#include <cstdio>
#include <utility>

class Vector
{
public:

  Vector() = default;
  explicit Vector(unsigned size) : s(size), v(new int[size])
  {}

  Vector(Vector const &o)
    : s(o.s), v(o.v ? new int[o.s] : nullptr)
  {
    if (!v)
      return;

    for (unsigned x = 0; x < s; ++x)
      v[x] = o.v[x];
  }

  Vector &operator = (Vector const &o)
  {
    if (this == &o)
      return *this;

    if (v)
      delete [] v;

    if (!o.v)
      {
        v = nullptr;
        return *this;
      }

    v = new int[o.s];
    s = o.s;

    for (unsigned x = 0; x < s; ++x)
      v[x] = o.v[x];

    return *this;
  }

  Vector(Vector &&o)
  : s(o.s), v(o.v)
  {
    printf("move ctor ... %p -> %p\n", &o, this);
    o.v = nullptr;
  }

  Vector &operator = (Vector &&o)
  {
    printf("move... %p -> %p\n", &o, this);
    if (this == &o)
      return *this;

    if (v)
      delete [] v;

    v = o.v;
    s = o.s;
    o.v = nullptr;
    return *this;
  }

  ~Vector()
  {
    if (v)
      delete [] v;
  }

  int const &at(unsigned idx) const
  { return v[idx]; }

  int &at(unsigned idx)
  { return v[idx]; }

  void info() const
  {
    printf("Vector: this=%p s=%d v=%p\n", this, s, v);
  }

private:
  unsigned s;
  int *v = nullptr;
};

int main()
{
  Vector u(10);
  u.info();

  Vector n;
  n = u;
  n.info();

  n = Vector(20);

  Vector n3(std::move(n));

  n.info();
  return 0;
}
